%
%  call stimuli() to generate stimuli for the FMO model
%
%  stimuli; stimuli(); stimuli( choice );
%
%  choice = 'current' or 'kaplan' or 'all' or 'paper' or
%    'dissertation' or 'interesting'
%

function stimulus = stimuli(choice)

    if (nargin == 0)
        create_current_stimulus();
        return 
    end

    if strcmp(choice, 'all') 
        create_all_stimuli();
    end

    if strcmp(choice, 'gena') 
        create_gena_stimuli();
    end

    if strcmp(choice, 'kaplan') 
        create_kaplan_stimuli();
    end

    if strcmp(choice, 'michotte') 
        create_michotte_stimuli();
    end

    if strcmp(choice, 'chimera') 
        create_chimera();
    end

    if strcmp(choice, 'motion-test') 
        stimulus = create_motion_test();
    end

    if strcmp(choice, 'moonwalk_flicker') 
        stimulus = create_moonwalk_flicker();
    end

    if strcmp(choice, 'moonwalk_static') 
        stimulus = create_moonwalk_static();
    end

    if strcmp(choice, 'edge_flicker') 
        stimulus = create_edge_flicker();
    end

    if strcmp(choice, 'edge_static') 
        stimulus = create_edge_static();
    end

    if strcmp(choice, 'full_motion') 
        stimulus = create_full_motion();
    end

    if strcmp(choice, 'paper') 
        create_barnes_mingolla_2013_stimuli();
    end

    if strcmp(choice, 'dissertation') 
        create_tim_barnes_dissertation_stimuli();
    end

    if strcmp(choice, 'interesting') 
        create_interesting_stimuli();
    end    
    
    if strcmp(choice, 'current') 
        create_current_stimulus();
    end

return;

%
%  create_current_stimulus()
%

function create_current_stimulus

    parameters = default_stimulus_parameters();
    
    parameters.saving.filename = 'kaplan_gray_deletion_moving';

    parameters.type = 'kaplan';
    parameters.boundary_velocity = [1 0];

    parameters.fg_texture = 'grayscale';
    parameters.fg_velocity = [1 0];

    parameters.bg_texture = 'grayscale';
    parameters.bg_velocity = [0 0];

    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);

return;

% need moonwalk_flicker, moonwalk_static, edge_flicker, edge_static

function stimulus = create_moonwalk_flicker
    
    parameters = default_stimulus_parameters();
    parameters.saving.filename = 'moonwalk_flicker';
    
    parameters.type = 'aperture';
    parameters.boundary_velocity = [0 0];
    
    parameters.fg_texture = 'grayscale';
    parameters.fg_velocity = [1 0];
    
    parameters.bg_texture = 'grayscale_flicker';
    parameters.bg_velocity = [0 0];
    
    stimulus = create_fmo_input(make_stimulus(parameters));
    save_all(stimulus, parameters.saving);
    
return;

function stimulus = create_moonwalk_static
    
    parameters = default_stimulus_parameters();
    parameters.saving.filename = 'moonwalk_static';
    
    parameters.type = 'aperture';
    parameters.boundary_velocity = [0 0];
    
    parameters.fg_texture = 'grayscale';
    parameters.fg_velocity = [1 0];
    
    parameters.bg_texture = 'grayscale';
    parameters.bg_velocity = [0 0];
    
    stimulus = create_fmo_input(make_stimulus(parameters));
    save_all(stimulus, parameters.saving);
    
return;

function stimulus = create_edge_flicker
    
    parameters = default_stimulus_parameters();
    parameters.type = 'kaplan';
    parameters.fg_velocity = [1 0];
    parameters.bg_velocity = [0 0];
    parameters.boundary_velocity = [0 0];
    
    parameters.saving.filename = 'edge_bw_flicker';
    parameters.fg_texture = 'dense';
    parameters.bg_texture = 'dense-flicker';
    stimulus = create_fmo_input(make_stimulus(parameters));
    save_all(stimulus, parameters.saving);
    
    parameters.saving.filename = 'edge_gray_flicker';
    parameters.fg_texture = 'grayscale';
    parameters.bg_texture = 'grayscale-flicker';
    stimulus = create_fmo_input(make_stimulus(parameters));
    save_all(stimulus, parameters.saving);
    
return;

function stimulus = create_edge_static
    
    parameters = default_stimulus_parameters();
    parameters.saving.filename = 'edge_static';
    
    parameters.type = 'kaplan';
    parameters.boundary_velocity = [0 0];
    
    parameters.fg_texture = 'grayscale';
    parameters.fg_velocity = [1 0];
    
    parameters.bg_texture = 'grayscale';
    parameters.bg_velocity = [0 0];
    
    stimulus = create_fmo_input(make_stimulus(parameters));
    save_all(stimulus, parameters.saving);
    
return;

function stimulus = create_full_motion
    
    parameters = default_stimulus_parameters();
    parameters.saving.filename = 'full_motion';
    
    parameters.type = 'foreground_only';
    parameters.boundary_velocity = [0 0];
    
    parameters.fg_texture = 'grayscale';
    parameters.fg_velocity = [1 0];
    
    parameters.bg_texture = 'grayscale';
    parameters.bg_velocity = [0 0];
    
    stimulus = create_fmo_input(make_stimulus(parameters));
    save_all(stimulus, parameters.saving);
    
return;


function stimulus = create_motion_test

    parameters = default_stimulus_parameters();
    parameters.n_frames = 21;
    
    parameters.type = 'kaplan';
    parameters.fg_texture = 'checkerboard';
    parameters.bg_texture = 'checkerboard';
    parameters.dot_size = 5;

    % left side moving
    
    parameters.saving.filename = 'motion_test1';
    parameters.boundary_velocity = [0 0];
    parameters.fg_velocity = [1 0];
    parameters.bg_velocity = [0 0];
    stimulus = create_fmo_input(make_stimulus(parameters));
    save_all(stimulus, parameters.saving);
    
    parameters.saving.filename = 'motion_test2';
    parameters.boundary_velocity = [0 0];
    parameters.fg_velocity = [1 1];
    parameters.bg_velocity = [0 0];
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);
    
return;

function stimulus = create_chimera
    
    parameters = default_stimulus_parameters();
    parameters.saving.filename = 'chimera';
    
    parameters.type = 'kaplan';
    parameters.boundary_velocity = [0 0];
    
    parameters.fg_texture = 'dense';
    parameters.fg_velocity = [1 0];
    
    parameters.bg_texture = 'dense';
    parameters.bg_velocity = [1 0];
    
    stimulus = create_fmo_input(make_stimulus(parameters));
    save_all(stimulus, parameters.saving);
    
return;

%
%   Create all the stimuli for testing with Gena's implementation
%

function create_gena_stimuli

% create each stimulus
% each is a sequence of 8 64x64 png images

    parameters = default_stimulus_parameters();
    
    parameters.image_size = 64;
    parameters.n_frames = 8;
    
    parameters.saving.directory = 'sequences/';
    parameters.saving.save_mat = true;
    parameters.saving.save_all_frames = true;
    parameters.saving.save_movies = true;
    
    % kaplan black & white - 4 cases
    
    parameters.type = 'kaplan';
    parameters.fg_texture = 'dense';
    parameters.bg_texture = 'dense';
    parameters.bg_velocity = [0 0];
    
    parameters.saving.filename = 'kaplan_bw_accretion_stationary';
    parameters.boundary_velocity = [0 0];
    parameters.fg_velocity = [-1 0];
    save_all(create_fmo_input(make_stimulus(parameters)), parameters.saving);
    
    parameters.saving.filename = 'kaplan_bw_accretion_moving';
    parameters.boundary_velocity = [-1 0];
    save_all(create_fmo_input(make_stimulus(parameters)), parameters.saving);
    
    parameters.saving.filename = 'kaplan_bw_deletion_stationary';
    parameters.boundary_velocity = [0 0];
    parameters.fg_velocity = [1 0];
    save_all(create_fmo_input(make_stimulus(parameters)), parameters.saving);
    
    parameters.saving.filename = 'kaplan_bw_deletion_moving';
    parameters.boundary_velocity = [1 0];
    save_all(create_fmo_input(make_stimulus(parameters)), parameters.saving);
    
    % kaplan grayscale - 4 cases
    
    parameters.type = 'kaplan';
    parameters.fg_texture = 'grayscale';
    parameters.bg_texture = 'grayscale';
    parameters.bg_velocity = [0 0];
    
    parameters.saving.filename = 'kaplan_gray_accretion_stationary';
    parameters.boundary_velocity = [0 0];
    parameters.fg_velocity = [-1 0];
    save_all(create_fmo_input(make_stimulus(parameters)), parameters.saving);
    
    parameters.saving.filename = 'kaplan_gray_accretion_moving';
    parameters.boundary_velocity = [-1 0];
    save_all(create_fmo_input(make_stimulus(parameters)), parameters.saving);
    
    parameters.saving.filename = 'kaplan_gray_deletion_stationary';
    parameters.boundary_velocity = [0 0];
    parameters.fg_velocity = [1 0];
    save_all(create_fmo_input(make_stimulus(parameters)), parameters.saving);
    
    parameters.saving.filename = 'kaplan_gray_deletion_moving';
    parameters.boundary_velocity = [1 0];
    save_all(create_fmo_input(make_stimulus(parameters)), parameters.saving);
    
    % royden - ? cases
    
    parameters.saving.filename = 'royden';
    
    
    
    % black & white circle - 6 cases (fg, bg, fg&bg opposite, moving circle)
    
    parameters.type = 'aperture';
    parameters.fg_texture = 'dense';
    parameters.bg_texture = 'dense';
    parameters.bg_velocity = [0 0];
    
    parameters.saving.filename = 'bw_circle_fg_right_stationary';
    
    
    parameters.saving.filename = 'bw_circle_bg_left_stationary';
    
    
    parameters.saving.filename = 'bw_circle_fg_right_bg_left_stationary';
    
    
    parameters.saving.filename = 'bw_circle_fg_right_moving_right';
    
    
    parameters.saving.filename = 'bw_circle_fg_right_moving_left';
    
    
    parameters.saving.filename = 'bw_circle_fg_right_moving_down';

    
    % circle grayscale - 3 cases
    
    parameters.type = 'aperture';
    parameters.fg_texture = 'grayscale';
    parameters.bg_texture = 'grayscale';
    parameters.bg_velocity = [0 0];
    
    parameters.saving.filename = 'gray_circle_fg_right_stationary';
    
    
    parameters.saving.filename = 'gray_circle_bg_left_stationary';
    
    
    parameters.saving.filename = 'gray_circle_fg_right_bg_left_stationary';
    
    
    parameters.saving.filename = 'gray_circle_fg_right_moving_right';
    
    
    parameters.saving.filename = 'gray_circle_fg_right_moving_left';
    
    
    parameters.saving.filename = 'gray_circle_fg_right_moving_down';

    % michotte texture binary
    
    % for michotte, sample the middle of the sequence while bout half the
    % disk is diappearing
    
    parameters.saving.filename = 'michotte_texture_bw';
    
    
    % michotte texture grayscale
    
    parameters.saving.filename = 'michotte_texture_gray';
    
    
    % michotte lunminance
    
    parameters.saving.filename = 'michotte_luminance_bw';
    
    
    parameters.saving.filename = 'michotte_luminance_gray';
    
    
    % --- not generated ---
    
    % optic flow database grayscale selections - 4 example?
    
    parameters.saving.filename = 'optic_flow_db_example1';
    
    
    % grayscale movie selections - 4 examples
    
    parameters.saving.filename = 'gray_movie_sign';

    

return;


%
%  create_interesting_stimuli()
%

function create_interesting_stimuli

    parameters = default_stimulus_parameters();
    parameters.saving.filename = 'kaplan_gray_deletion_moving';

    parameters.type = 'kaplan';
    parameters.boundary_velocity = [1 0];

    parameters.fg_texture = 'grayscale';
    parameters.fg_velocity = [1 0];

    parameters.bg_texture = 'grayscale';
    parameters.bg_velocity = [0 0];

    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);

    % chimera
    
    parameters.saving.filename = 'kaplan_chimera';
    parameters.type = 'kaplan';
    parameters.fg_texture = 'dense';
    parameters.bg_texture = 'dense';
    parameters.fg_velocity = [1 0];
    parameters.bg_velocity = [1 0];
    parameters.boundary_velocity = [0 0];
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);
    
    % others?
    
return;

%
%  create_kaplan_stimuli()
%
%  5 stimuli, deletion, accretion x stationary, moving + shear
%

function create_kaplan_stimuli

    parameters = default_stimulus_parameters();

    parameters.n_frames = 21;
    
    parameters.type = 'kaplan';
    parameters.fg_texture = 'dense';
    parameters.bg_texture = 'dense';

    % accretion
    
    parameters.saving.filename = 'kaplan_bw_accretion_static';
    parameters.boundary_velocity = [0 0];
    parameters.fg_velocity = [0 0];
    parameters.bg_velocity = [1 0];
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);

    % deletion
    
    parameters.saving.filename = 'kaplan_bw_deletion_static';
    parameters.boundary_velocity = [0 0];
    parameters.fg_velocity = [1 0];
    parameters.bg_velocity = [0 0];
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);

    % accretion, moving boundary
    
    parameters.saving.filename = 'kaplan_bw_accretion_moving';
    parameters.boundary_velocity = [1 0];
    parameters.fg_velocity = [0 0];
    parameters.bg_velocity = [1 0];
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);

    % deletion, moving boundary 
    
    parameters.saving.filename = 'kaplan_bw_deletion_moving';
    parameters.boundary_velocity = [1 0];
    parameters.fg_velocity = [1 0];
    parameters.bg_velocity = [0 0];
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);

    % shear
    
    parameters.type = 'top';
    
    parameters.saving.filename = 'kaplan_bw_shear';
    parameters.boundary_velocity = [0 0];
    parameters.fg_velocity = [0 0];
    parameters.bg_velocity = [1 0];
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);
    
    %
    % now grayscale
    %
    
    parameters.type = 'kaplan';
    parameters.fg_texture = 'grayscale';
    parameters.bg_texture = 'grayscale';

    % accretion
    
    parameters.saving.filename = 'kaplan_gray_accretion_static';
    parameters.boundary_velocity = [0 0];
    parameters.fg_velocity = [0 0]; % left side
    parameters.bg_velocity = [1 0]; % right side
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);

    % deletion
    
    parameters.saving.filename = 'kaplan_gray_deletion_static';
    parameters.boundary_velocity = [0 0];
    parameters.fg_velocity = [1 0]; % left side
    parameters.bg_velocity = [0 0]; % right side
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);

    % accretion, moving boundary
    
    parameters.saving.filename = 'kaplan_gray_accretion_moving';
    parameters.boundary_velocity = [1 0];
    parameters.fg_velocity = [0 0];
    parameters.bg_velocity = [1 0];
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);

    % deletion, moving boundary 
    
    parameters.saving.filename = 'kaplan_gray_deletion_moving';
    parameters.boundary_velocity = [1 0];
    parameters.fg_velocity = [1 0];
    parameters.bg_velocity = [0 0];
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);
    
    %  shear
    
    parameters.type = 'top';
    
    parameters.saving.filename = 'kaplan_gray_shear';
    parameters.boundary_velocity = [0 0];
    parameters.fg_velocity = [0 0];
    parameters.bg_velocity = [1 0];
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);

return;

%
%  create_michotte_stimuli()
%

function create_michotte_stimuli
    
    parameters = default_stimulus_parameters();
    
    parameters.image_size = 256;
    parameters.n_frames = 41 + 192;
    parameters.saving.fps = 60;
    
    parameters.saving.save_movies = 0;
    parameters.image_size = 64;
    parameters.n_frames = 41;
    
    parameters.type = 'michotte';
    parameters.boundary_velocity = [1 0];
    parameters.fg_velocity = [1 0];
    
    % texture version
    
    parameters.saving.filename = 'michotte-dense';
    parameters.fg_texture = 'dense';
    parameters.bg_texture = 'dense';
%    in = create_fmo_input(make_stimulus(parameters));
%    save_all(in, parameters.saving);
    
    parameters.saving.filename = 'michotte-grayscale';
    parameters.fg_texture = 'grayscale';
    parameters.bg_texture = 'grayscale';
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);

    % black disk on white background
    
    parameters.saving.filename = 'michotte-standard';
    parameters.fg_texture = 'black';
    parameters.bg_texture = 'white';
    in = create_fmo_input(make_stimulus(parameters));
    save_all(in, parameters.saving);

return;

%
%  create_barnes_mingolla_2013_stimuli()
%

function create_barnes_mingolla_2013_stimuli

% look in paper



return;

%
%  create_tim_barnes_dissertation_stimuli()
%

function create_tim_barnes_dissertation_stimuli

% look in dissertation



return;

%
%  create_all_stimuli()
%

function create_all_stimuli

    s.saving.directory = 'movies/';
    s.saving.save_mat = true;
    s.saving.save_all_frames = false;
    s.saving.save_movies = true;

    s.type = 'kaplan'; % michotte, kaplan, royden, koenderink
    s.fg_texture = 'dense'; % dense, random, white, gray, black
    s.bg_texture = 'dense'; % dense, random, white, gray, black
    s.sparse_regular.spacing = 7;
    s.fg_velocity = [1 0];
    s.bg_velocity = [0 0];
    s.boundary_velocity = [0 0];
    s.dot_size = 2;
    s.dot_spacing = 1 * s.dot_size;
    s.dot_shape = 'circle'; % circle, square, diamond

    s.saving.filename = 'kaplan_shear';
    s.fg_velocity = [0 -1];
    in = create_fmo_input( make_stimulus(s) );
    save_all(in, s.saving);

    % delete_stat needs to have fg_velocity set if I keep what's above
    s.saving.filename = 'kaplan_delete_stat';
    in = create_fmo_input( make_stimulus(s) );
    save_all(in, s.saving);

    s.saving.filename = 'kaplan_accrete_stat';
    s.fg_velocity = [-1 0];
    in = create_fmo_input( make_stimulus(s) );
    save_all(in, s.saving);

    s.saving.filename = 'kaplan_accrete_mov';
    s.boundary_velocity = [-1 0];
    in = create_fmo_input( make_stimulus(s) );
    save_all(in, s.saving);

    s.saving.filename = 'kaplan_delete_mov';
    s.fg_velocity = [1 0];
    s.boundary_velocity = [1 0];
    in = create_fmo_input( make_stimulus(s) );
    save_all(in, s.saving);

    s.saving.filename = 'kaplan_delete_diag';
    s.fg_velocity = [0 1];
    s.boundary_velocity = [0 0];
    in = create_fmo_input( make_stimulus(s) );
    save_all(in, s.saving);

    s.saving.filename = 'royden_shearing';
    s.type = 'royden';
    s.fg_velocity = [1 0];
    s.royden.aspect_ratio = 2;
    in = create_fmo_input( make_stimulus(s) );
    save_all(in, s.saving);

    s.saving.filename = 'royden_occluding';
    s.royden.aspect_ratio = 0.5;
    in = create_fmo_input( make_stimulus(s) );
    save_all(in, s.saving);

    s.saving.filename = 'michotte';
    s.type = 'michotte';
    s.boundary_velocity = [1 0];
    in = create_fmo_input( make_stimulus(s) );
    save_all(in, s.saving);

    s.saving.filename = 'koenderink';
    s.type = 'koenderink';
    s.boundary_velocity = [0 0];
    in = create_fmo_input( make_stimulus(s) );
    save_all(in, s.saving);

return;

%
%  create_fmo_input( stimulus )
%

function in = create_fmo_input (stimulus)

    %% this fade stuff is not used, so I commented it out for now
    %% 
    %n_fade = 20; %12
    %fade_size = size(stimulus(:,:,1));
    %rows = (1:fade_size(1))' * ones(1, fade_size(2)) / n_fade;
    %cols = ones(fade_size(1), 1) * (1:fade_size(2)) / n_fade;
    %fade = min(min(min(min(ones(fade_size), rows), cols), ...
    %                       rows(end:-1:1, :)), cols(:, end:-1:1));

    edge = [-1, -1, -1; -1, 8, -1; -1, -1, -1] / 16;

    for i = 1 : size(stimulus, 3)
        in.orig(:, :, i) = 0.5 + (stimulus(:, :, i) - 0.5); % .* fade;

        lgn = conv2 (in.orig(:, :, i), edge, 'same');    
        in.form_on(:, :, i)  =  max (lgn, 0);
        in.form_off(:, :, i) = -min (lgn, 0);

        if i == 1
            temp = zeros(size(in.orig(:,:,1)));
        else
            temp = in.orig(:,:,i) - in.orig(:,:,i-1);
        end
        in.motion_on(:,:,i) = max (temp, 0);
        in.motion_off(:,:,i) = max (-temp, 0);
    end

return

%
%  save_all ( stimulus, save-parameters )
%

function save_all(in, saving)

    % save stimulus
    
    if (saving.save_mat)
        save(sprintf('%s.mat', saving.filename), 'in')
    end

    if (exist(saving.directory, 'dir') == 0)
        mkdir(saving.directory);
    end 

    n_frames = size(in.form_on, 3);
    fields = fieldnames(in);

    % save as a movie
    
    if (saving.save_movies)
        for j = 1:length (fields)

            movieFileName = [saving.directory saving.filename '_' fields{j} '_movie.avi'];
            movieFileObject = VideoWriter(movieFileName);
            movieFileObject.FrameRate = saving.fps;
            open(movieFileObject);

            for i = 1:n_frames
                image = eval(['in.' fields{j}]);
                writeVideo(movieFileObject, image(:,:,i));
            end

            close(movieFileObject);
        end
    end

    % save as individual images
    
    if (saving.save_all_frames)
        subdirectory = [saving.directory saving.filename '/'];

        if (exist(subdirectory, 'dir') == 0)
            mkdir(subdirectory);
        end 

        for j = 1:length (fields)
            for i = 1:n_frames
                cmd = sprintf ('imwrite (in.%s(:,:,%d), ''%s%s%03d.png''', ...
                               fields{j}, i, subdirectory, fields{j}, i-1);
                cmd = [ cmd ', ''png'', ''bitdepth'', 8)'];
                eval (cmd)
            end
        end
    end

return
